"""Tools management commands."""

import click
from rich.table import Table
from rich.syntax import Syntax

from ..utils.output import console


@click.group(name="tools")
def tools_group():
    """Manage Hanzo tools and plugins."""
    pass


@tools_group.command(name="list")
@click.option("--category", "-c", help="Filter by category")
@click.option("--installed", is_flag=True, help="Show only installed tools")
@click.pass_context
async def list_tools(ctx, category: str, installed: bool):
    """List available tools."""
    try:
        from hanzo_tools import get_tool_registry
    except ImportError:
        console.print("[red]Error:[/red] hanzo-tools not installed")
        console.print("Install with: pip install hanzo[tools]")
        return

    registry = get_tool_registry()

    with console.status("Loading tools..."):
        try:
            tools = await registry.list_tools(
                category=category, installed_only=installed
            )
        except Exception as e:
            console.print(f"[red]Failed to load tools: {e}[/red]")
            return

    if not tools:
        console.print("[yellow]No tools found[/yellow]")
        return

    # Group by category
    categories = {}
    for tool in tools:
        cat = tool.get("category", "uncategorized")
        if cat not in categories:
            categories[cat] = []
        categories[cat].append(tool)

    # Display tools
    for cat, cat_tools in sorted(categories.items()):
        table = Table(title=f"{cat.title()} Tools")
        table.add_column("Name", style="cyan")
        table.add_column("Version", style="green")
        table.add_column("Description", style="white")
        table.add_column("Status", style="yellow")

        for tool in sorted(cat_tools, key=lambda t: t["name"]):
            table.add_row(
                tool["name"],
                tool.get("version", "latest"),
                tool.get("description", ""),
                "installed" if tool.get("installed") else "available",
            )

        console.print(table)
        if len(categories) > 1:
            console.print()


@tools_group.command()
@click.argument("tool_name")
@click.option("--version", "-v", help="Specific version to install")
@click.pass_context
async def install(ctx, tool_name: str, version: str):
    """Install a tool."""
    try:
        from hanzo_tools import get_tool_registry
    except ImportError:
        console.print("[red]Error:[/red] hanzo-tools not installed")
        return

    registry = get_tool_registry()

    with console.status(f"Installing {tool_name}..."):
        try:
            result = await registry.install_tool(name=tool_name, version=version)

            console.print(
                f"[green]✓[/green] Installed {tool_name} v{result['version']}"
            )

            if deps := result.get("dependencies_installed"):
                console.print(f"  Dependencies: {', '.join(deps)}")

            if config := result.get("post_install_message"):
                console.print(f"\n[yellow]Configuration:[/yellow]")
                console.print(config)

        except Exception as e:
            console.print(f"[red]Failed to install {tool_name}: {e}[/red]")


@tools_group.command()
@click.argument("tool_name")
@click.pass_context
async def uninstall(ctx, tool_name: str):
    """Uninstall a tool."""
    try:
        from hanzo_tools import get_tool_registry
    except ImportError:
        console.print("[red]Error:[/red] hanzo-tools not installed")
        return

    registry = get_tool_registry()

    if click.confirm(f"Uninstall {tool_name}?"):
        with console.status(f"Uninstalling {tool_name}..."):
            try:
                await registry.uninstall_tool(tool_name)
                console.print(f"[green]✓[/green] Uninstalled {tool_name}")
            except Exception as e:
                console.print(f"[red]Failed to uninstall {tool_name}: {e}[/red]")


@tools_group.command()
@click.argument("tool_name")
@click.option("--version", "-v", help="Target version")
@click.pass_context
async def update(ctx, tool_name: str, version: str):
    """Update a tool."""
    try:
        from hanzo_tools import get_tool_registry
    except ImportError:
        console.print("[red]Error:[/red] hanzo-tools not installed")
        return

    registry = get_tool_registry()

    with console.status(f"Updating {tool_name}..."):
        try:
            result = await registry.update_tool(name=tool_name, version=version)

            console.print(f"[green]✓[/green] Updated {tool_name}")
            console.print(f"  Previous: v{result['previous_version']}")
            console.print(f"  Current: v{result['current_version']}")

        except Exception as e:
            console.print(f"[red]Failed to update {tool_name}: {e}[/red]")


@tools_group.command()
@click.argument("tool_name")
@click.pass_context
async def info(ctx, tool_name: str):
    """Show tool information."""
    try:
        from hanzo_tools import get_tool_registry
    except ImportError:
        console.print("[red]Error:[/red] hanzo-tools not installed")
        return

    registry = get_tool_registry()

    with console.status(f"Loading {tool_name} info..."):
        try:
            info = await registry.get_tool_info(tool_name)
        except Exception as e:
            console.print(f"[red]Failed to get info: {e}[/red]")
            return

    console.print(f"[cyan]{info['name']}[/cyan]")
    console.print(f"  Version: {info['version']}")
    console.print(f"  Category: {info['category']}")
    console.print(f"  Author: {info.get('author', 'Unknown')}")
    console.print(f"  License: {info.get('license', 'Unknown')}")

    if desc := info.get("description"):
        console.print(f"\n{desc}")

    if features := info.get("features"):
        console.print("\n[cyan]Features:[/cyan]")
        for feature in features:
            console.print(f"  • {feature}")

    if deps := info.get("dependencies"):
        console.print("\n[cyan]Dependencies:[/cyan]")
        for dep in deps:
            console.print(f"  • {dep}")

    if usage := info.get("usage_example"):
        console.print("\n[cyan]Usage Example:[/cyan]")
        syntax = Syntax(usage, "python", theme="monokai", line_numbers=False)
        console.print(syntax)


@tools_group.command()
@click.argument("tool_name")
@click.argument("args", nargs=-1)
@click.option("--json", "-j", is_flag=True, help="Output as JSON")
@click.pass_context
async def run(ctx, tool_name: str, args: tuple, json: bool):
    """Run a tool directly."""
    try:
        from hanzo_tools import run_tool
    except ImportError:
        console.print("[red]Error:[/red] hanzo-tools not installed")
        return

    # Parse arguments
    tool_args = {}
    for arg in args:
        if "=" in arg:
            key, value = arg.split("=", 1)
            tool_args[key] = value
        else:
            console.print(f"[red]Invalid argument format: {arg}[/red]")
            console.print("Use: key=value")
            return

    with console.status(f"Running {tool_name}..."):
        try:
            result = await run_tool(name=tool_name, args=tool_args)

            if json:
                console.print_json(data=result)
            else:
                if isinstance(result, str):
                    console.print(result)
                elif isinstance(result, dict):
                    for key, value in result.items():
                        console.print(f"{key}: {value}")
                else:
                    console.print(result)

        except Exception as e:
            console.print(f"[red]Tool execution failed: {e}[/red]")


@tools_group.command()
@click.option("--check", is_flag=True, help="Check for updates only")
@click.pass_context
async def upgrade(ctx, check: bool):
    """Upgrade all tools."""
    try:
        from hanzo_tools import get_tool_registry
    except ImportError:
        console.print("[red]Error:[/red] hanzo-tools not installed")
        return

    registry = get_tool_registry()

    with console.status("Checking for updates..."):
        try:
            updates = await registry.check_updates()
        except Exception as e:
            console.print(f"[red]Failed to check updates: {e}[/red]")
            return

    if not updates:
        console.print("[green]✓[/green] All tools are up to date")
        return

    # Show available updates
    table = Table(title="Available Updates")
    table.add_column("Tool", style="cyan")
    table.add_column("Current", style="yellow")
    table.add_column("Latest", style="green")
    table.add_column("Changes", style="white")

    for update in updates:
        table.add_row(
            update["name"],
            update["current_version"],
            update["latest_version"],
            update.get("changelog_summary", ""),
        )

    console.print(table)

    if check:
        return

    # Perform updates
    if click.confirm(f"Update {len(updates)} tools?"):
        for update in updates:
            with console.status(f"Updating {update['name']}..."):
                try:
                    await registry.update_tool(update["name"])
                    console.print(f"[green]✓[/green] Updated {update['name']}")
                except Exception as e:
                    console.print(f"[red]Failed to update {update['name']}: {e}[/red]")


@tools_group.command()
@click.argument("name")
@click.option("--template", "-t", help="Tool template to use")
@click.pass_context
async def create(ctx, name: str, template: str):
    """Create a new custom tool."""
    try:
        from hanzo_tools import create_tool_template
    except ImportError:
        console.print("[red]Error:[/red] hanzo-tools not installed")
        return

    with console.status(f"Creating tool '{name}'..."):
        try:
            path = await create_tool_template(name=name, template=template or "basic")

            console.print(f"[green]✓[/green] Created tool template at: {path}")
            console.print("\nNext steps:")
            console.print("1. Edit the tool implementation")
            console.print("2. Test with: hanzo tools run {name}")
            console.print("3. Package with: hanzo tools package {name}")

        except Exception as e:
            console.print(f"[red]Failed to create tool: {e}[/red]")



"""Chat command for interactive AI conversations."""

import asyncio
from typing import Optional

import click
import httpx
from rich.markdown import Markdown

from ..utils.output import console


@click.command(name="chat")
@click.option("--model", "-m", default="llama-3.2-3b", help="Model to use")
@click.option("--local/--cloud", default=True, help="Use local or cloud model")
@click.option("--once", is_flag=True, help="Single question mode")
@click.option("--system", "-s", help="System prompt")
@click.option(
    "--repl", is_flag=True, help="Start full REPL interface (like Claude Code)"
)
@click.option("--ipython", is_flag=True, help="Use IPython REPL interface")
@click.option("--tui", is_flag=True, help="Use beautiful TUI interface")
@click.argument("prompt", nargs=-1)
@click.pass_context
def chat_command(
    ctx,
    model: str,
    local: bool,
    once: bool,
    system: Optional[str],
    repl: bool,
    ipython: bool,
    tui: bool,
    prompt: tuple,
):
    """Interactive AI chat."""
    # Check if REPL mode requested
    if repl or ipython or tui:
        try:
            import os
            import sys

            # Set up environment
            if model:
                os.environ["HANZO_DEFAULT_MODEL"] = model
            if local:
                os.environ["HANZO_USE_LOCAL"] = "true"
            if system:
                os.environ["HANZO_SYSTEM_PROMPT"] = system

            if ipython:
                from hanzo_repl.ipython_repl import main

                sys.exit(main())
            elif tui:
                from hanzo_repl.textual_repl import main

                sys.exit(main())
            else:
                from hanzo_repl.cli import main

                sys.exit(main())
        except ImportError:
            console.print("[red]Error:[/red] hanzo-repl not installed")
            console.print("Install with: pip install hanzo[repl]")
            console.print("\nAlternatively:")
            console.print("  pip install hanzo-repl")
            return

    prompt_text = " ".join(prompt) if prompt else None

    if once or prompt_text:
        # Single question mode
        asyncio.run(ask_once(ctx, prompt_text or "Hello", model, local, system))
    else:
        # Interactive chat
        asyncio.run(interactive_chat(ctx, model, local, system))


async def ask_once(
    ctx, prompt: str, model: str, local: bool, system: Optional[str] = None
):
    """Ask a single question."""
    messages = []
    if system:
        messages.append({"role": "system", "content": system})
    messages.append({"role": "user", "content": prompt})

    try:
        if local:
            # Use local cluster
            base_url = "http://localhost:8000"

            # Check if cluster is running
            try:
                async with httpx.AsyncClient() as client:
                    await client.get(f"{base_url}/health")
            except httpx.ConnectError:
                console.print(
                    "[yellow]Local cluster not running. Start with: hanzo serve[/yellow]"
                )
                return

            # Make request to local cluster
            async with httpx.AsyncClient() as client:
                response = await client.post(
                    f"{base_url}/v1/chat/completions",
                    json={"model": model, "messages": messages, "stream": False},
                )
                response.raise_for_status()
                result = response.json()
                content = result["choices"][0]["message"]["content"]
        else:
            # Use cloud API
            try:
                from hanzoai import completion

                result = completion(
                    model=f"anthropic/{model}" if "claude" in model else model,
                    messages=messages,
                )
                content = result.choices[0].message.content
            except ImportError:
                console.print("[red]Error:[/red] hanzoai not installed")
                console.print("Install with: pip install hanzo[all]")
                return

        # Display response
        if ctx.obj.get("json"):
            console.print_json(data={"response": content})
        else:
            console.print(Markdown(content))

    except Exception as e:
        console.print(f"[red]Error: {e}[/red]")


async def interactive_chat(ctx, model: str, local: bool, system: Optional[str]):
    """Run interactive chat session."""
    from prompt_toolkit import PromptSession
    from prompt_toolkit.history import FileHistory

    console.print(
        f"[cyan]Chat session started[/cyan] (model: {model}, mode: {'local' if local else 'cloud'})"
    )
    console.print("Type 'exit' or Ctrl+D to quit\n")

    session = PromptSession(history=FileHistory(".hanzo_chat_history"))
    messages = []

    if system:
        messages.append({"role": "system", "content": system})

    while True:
        try:
            # Get user input
            user_input = await session.prompt_async("You: ")

            if user_input.lower() in ["exit", "quit"]:
                break

            # Add to messages
            messages.append({"role": "user", "content": user_input})

            # Get response
            console.print("AI: ", end="")
            with console.status(""):
                if local:
                    # Use local cluster
                    async with httpx.AsyncClient() as client:
                        response = await client.post(
                            "http://localhost:8000/v1/chat/completions",
                            json={
                                "model": model,
                                "messages": messages,
                                "stream": False,
                            },
                        )
                        response.raise_for_status()
                        result = response.json()
                        content = result["choices"][0]["message"]["content"]
                else:
                    # Use cloud API
                    from hanzoai import completion

                    result = completion(
                        model=f"anthropic/{model}" if "claude" in model else model,
                        messages=messages,
                    )
                    content = result.choices[0].message.content

            # Display and save response
            console.print(Markdown(content))
            messages.append({"role": "assistant", "content": content})
            console.print()

        except KeyboardInterrupt:
            continue
        except EOFError:
            break
        except Exception as e:
            console.print(f"\n[red]Error: {e}[/red]\n")



"""Main CLI entry point for Hanzo."""

import os
import sys
import signal
import asyncio
import subprocess
from typing import Optional

import click
from rich.console import Console

from .commands import (
    mcp,
    auth,
    chat,
    repl,
    agent,
    miner,
    tools,
    config,
    cluster,
    network,
)
from .utils.output import console
from .interactive.repl import HanzoREPL

# Version
__version__ = "0.3.14"


@click.group(invoke_without_command=True)
@click.version_option(version=__version__, prog_name="hanzo")
@click.option("--verbose", "-v", is_flag=True, help="Verbose output")
@click.option("--json", is_flag=True, help="JSON output format")
@click.option("--config", "-c", type=click.Path(), help="Config file path")
@click.pass_context
def cli(ctx, verbose: bool, json: bool, config: Optional[str]):
    """Hanzo AI - Unified CLI for local, private, and free AI.

    Run without arguments to enter interactive mode.
    """
    # Ensure context object exists
    ctx.ensure_object(dict)
    ctx.obj["verbose"] = verbose
    ctx.obj["json"] = json
    ctx.obj["config"] = config
    ctx.obj["console"] = console

    # If no subcommand, enter interactive mode or start compute node
    if ctx.invoked_subcommand is None:
        # Check if we should start as a compute node
        import os

        if os.environ.get("HANZO_COMPUTE_NODE") == "1":
            # Start as a compute node

            asyncio.run(start_compute_node(ctx))
        else:
            # Enter interactive REPL mode
            console.print("[bold cyan]Hanzo AI - Interactive Mode[/bold cyan]")
            console.print("Type 'help' for commands, 'exit' to quit\n")
            try:
                repl = HanzoREPL(console=console)
                asyncio.run(repl.run())
            except KeyboardInterrupt:
                console.print("\n[yellow]Interrupted[/yellow]")
            except EOFError:
                console.print("\n[yellow]Goodbye![/yellow]")


# Register command groups
cli.add_command(agent.agent_group)
cli.add_command(auth.auth_group)
cli.add_command(cluster.cluster_group)
cli.add_command(mcp.mcp_group)
cli.add_command(miner.miner_group)
cli.add_command(chat.chat_command)
cli.add_command(repl.repl_group)
cli.add_command(tools.tools_group)
cli.add_command(network.network_group)
cli.add_command(config.config_group)


# Quick aliases
@cli.command()
@click.argument("prompt", nargs=-1, required=True)
@click.option("--model", "-m", default="llama-3.2-3b", help="Model to use")
@click.option("--local/--cloud", default=True, help="Use local or cloud model")
@click.pass_context
def ask(ctx, prompt: tuple, model: str, local: bool):
    """Quick question to AI (alias for 'hanzo chat --once')."""
    prompt_text = " ".join(prompt)
    asyncio.run(chat.ask_once(ctx, prompt_text, model, local))


@cli.command()
@click.option("--name", "-n", default="hanzo-local", help="Cluster name")
@click.option("--port", "-p", default=8000, help="API port")
@click.pass_context
def serve(ctx, name: str, port: int):
    """Start local AI cluster (alias for 'hanzo cluster start')."""
    asyncio.run(cluster.start_cluster(ctx, name, port))


@cli.command()
@click.option("--name", "-n", help="Node name (auto-generated if not provided)")
@click.option(
    "--port", "-p", default=52415, help="Node port (default: 52415 for hanzo/net)"
)
@click.option(
    "--network", default="local", help="Network to join (mainnet/testnet/local)"
)
@click.option(
    "--models", "-m", multiple=True, help="Models to serve (e.g., llama-3.2-3b)"
)
@click.option("--max-jobs", type=int, default=10, help="Max concurrent jobs")
@click.pass_context
def net(ctx, name: str, port: int, network: str, models: tuple, max_jobs: int):
    """Start the Hanzo Network distributed AI compute node."""
    try:
        asyncio.run(start_compute_node(ctx, name, port, network, models, max_jobs))
    except KeyboardInterrupt:
        # Already handled in start_compute_node
        pass


@cli.command()
@click.option("--name", "-n", help="Node name (auto-generated if not provided)")
@click.option(
    "--port", "-p", default=52415, help="Node port (default: 52415 for hanzo/net)"
)
@click.option(
    "--network", default="local", help="Network to join (mainnet/testnet/local)"
)
@click.option(
    "--models", "-m", multiple=True, help="Models to serve (e.g., llama-3.2-3b)"
)
@click.option("--max-jobs", type=int, default=10, help="Max concurrent jobs")
@click.pass_context
def node(ctx, name: str, port: int, network: str, models: tuple, max_jobs: int):
    """Alias for 'hanzo net' - Start as a compute node for the Hanzo network."""
    try:
        asyncio.run(start_compute_node(ctx, name, port, network, models, max_jobs))
    except KeyboardInterrupt:
        # Already handled in start_compute_node
        pass


@cli.command()
@click.option("--workspace", default="~/.hanzo/dev", help="Workspace directory")
@click.option(
    "--orchestrator",
    default="gpt-5",
    help="Orchestrator: gpt-5, router:gpt-4o, direct:claude, codex, gpt-5-pro-codex, cost-optimized",
)
@click.option(
    "--orchestrator-mode",
    type=click.Choice(["router", "direct", "codex", "hybrid", "local"]),
    default=None,
    help="Force orchestrator mode (router via hanzo-router, direct API, codex, hybrid, local)",
)
@click.option(
    "--router-endpoint",
    default=None,
    help="Hanzo router endpoint (default: http://localhost:4000)",
)
@click.option("--claude-path", help="Path to Claude Code executable")
@click.option("--monitor", is_flag=True, help="Start in monitor mode")
@click.option("--repl", is_flag=True, help="Start REPL interface (default)")
@click.option("--instances", type=int, default=2, help="Number of worker agents")
@click.option("--mcp-tools", is_flag=True, default=True, help="Enable all MCP tools")
@click.option(
    "--network-mode", is_flag=True, default=True, help="Network agents together"
)
@click.option(
    "--guardrails", is_flag=True, default=True, help="Enable code quality guardrails"
)
@click.option(
    "--use-network/--no-network", default=True, help="Use hanzo-network if available"
)
@click.option(
    "--use-hanzo-net",
    is_flag=True,
    help="Use hanzo/net for local AI (auto-enabled with local: models)",
)
@click.option(
    "--hanzo-net-port",
    type=int,
    default=52415,
    help="Port for hanzo/net (default: 52415)",
)
@click.pass_context
def dev(
    ctx,
    workspace: str,
    orchestrator: str,
    orchestrator_mode: str,
    router_endpoint: str,
    claude_path: str,
    monitor: bool,
    repl: bool,
    instances: int,
    mcp_tools: bool,
    network_mode: bool,
    guardrails: bool,
    use_network: bool,
    use_hanzo_net: bool,
    hanzo_net_port: int,
):
    """Start Hanzo Dev - AI Coding OS with configurable orchestrator.

    This creates a multi-agent system where:
    - Configurable orchestrator (GPT-5, GPT-4, Claude, or LOCAL) manages the network
    - Local AI via hanzo/net for cost-effective orchestration
    - Worker agents (Claude + local) handle code implementation
    - Critic agents review and improve code (System 2 thinking)
    - Cost-optimized routing (local models for simple tasks)
    - All agents can use MCP tools
    - Agents can call each other recursively
    - Guardrails prevent code degradation
    - Auto-recovery from failures

    Examples:
        hanzo dev                                    # GPT-5 orchestrator (default)
        hanzo dev --orchestrator gpt-4               # GPT-4 orchestrator
        hanzo dev --orchestrator claude-3-5-sonnet   # Claude orchestrator
        hanzo dev --orchestrator local:llama3.2      # Local Llama 3.2 via hanzo/net
        hanzo dev --use-hanzo-net                    # Enable local AI workers
        hanzo dev --instances 4                      # More worker agents
        hanzo dev --monitor                          # Auto-monitor and restart mode
    """
    from .dev import run_dev_orchestrator
    from .orchestrator_config import OrchestratorMode, get_orchestrator_config

    # Get orchestrator configuration
    orch_config = get_orchestrator_config(orchestrator)

    # Override mode if specified
    if orchestrator_mode:
        orch_config.mode = OrchestratorMode(orchestrator_mode)

    # Override router endpoint if specified
    if router_endpoint and orch_config.router:
        orch_config.router.endpoint = router_endpoint

    # Auto-enable hanzo net if using local orchestrator
    if orchestrator.startswith("local:") or orch_config.mode == OrchestratorMode.LOCAL:
        use_hanzo_net = True

    # Show configuration
    console.print(f"[bold cyan]Orchestrator Configuration[/bold cyan]")
    console.print(f"  Mode: {orch_config.mode.value}")
    console.print(f"  Primary Model: {orch_config.primary_model}")
    if orch_config.router:
        console.print(f"  Router Endpoint: {orch_config.router.endpoint}")
    if orch_config.codex:
        console.print(f"  Codex Model: {orch_config.codex.model}")
    console.print(
        f"  Cost Optimization: {'Enabled' if orch_config.enable_cost_optimization else 'Disabled'}"
    )
    console.print()

    asyncio.run(
        run_dev_orchestrator(
            workspace=workspace,
            orchestrator_model=orchestrator,
            orchestrator_config=orch_config,  # Pass the config
            claude_path=claude_path,
            monitor=monitor,
            repl=repl or not monitor,  # Default to REPL if not monitoring
            instances=instances,
            mcp_tools=mcp_tools,
            network_mode=network_mode,
            guardrails=guardrails,
            use_network=use_network,
            use_hanzo_net=use_hanzo_net,
            hanzo_net_port=hanzo_net_port,
            console=ctx.obj.get("console", console),
        )
    )


async def start_compute_node(
    ctx,
    name: str = None,
    port: int = 52415,
    network: str = "mainnet",
    models: tuple = None,
    max_jobs: int = 10,
):
    """Start this instance as a compute node using hanzo/net."""
    from .utils.net_check import check_net_installation

    console = ctx.obj.get("console", Console())

    console.print("[bold cyan]Starting Hanzo Net Compute Node[/bold cyan]")
    console.print(f"Network: {network}")
    console.print(f"Port: {port}")

    # Check hanzo/net availability
    is_available, net_path, python_exe = check_net_installation()

    if not is_available:
        console.print("[red]Error:[/red] hanzo-net is not installed")
        console.print("\nTo install hanzo-net from PyPI:")
        console.print("  pip install hanzo-net")
        console.print("\nOr for development, clone from GitHub:")
        console.print("  git clone https://github.com/hanzoai/net.git ~/work/hanzo/net")
        console.print("  cd ~/work/hanzo/net && pip install -e .")
        return

    try:
        import os
        import sys
        import subprocess

        # Use the checked net_path and python_exe
        if not net_path:
            # net is installed as a package
            console.print("[green]✓[/green] Using installed hanzo/net")

            # Set up sys.argv for net's argparse
            original_argv = sys.argv.copy()
            try:
                # Build argv for net
                sys.argv = ["hanzo-net"]  # Program name

                # Add options
                if port != 52415:
                    sys.argv.extend(["--chatgpt-api-port", str(port)])
                if name:
                    sys.argv.extend(["--node-id", name])
                if network != "local":
                    sys.argv.extend(["--discovery-module", network])
                if models:
                    sys.argv.extend(["--default-model", models[0]])

                # Import and run net
                from net.main import run as net_run

                console.print(f"\n[green]✓[/green] Node initialized")
                console.print(f"  Port: {port}")
                console.print(
                    f"  Models: {', '.join(models) if models else 'auto-detect'}"
                )
                console.print("\n[bold green]Hanzo Net is running![/bold green]")
                console.print("WebUI: http://localhost:52415")
                console.print("API: http://localhost:52415/v1/chat/completions")
                console.print("\nPress Ctrl+C to stop\n")

                # Set up signal handlers for async version
                stop_event = asyncio.Event()

                def async_signal_handler(signum, frame):
                    console.print("\n[yellow]Stopping hanzo net...[/yellow]")
                    stop_event.set()

                signal.signal(signal.SIGINT, async_signal_handler)
                signal.signal(signal.SIGTERM, async_signal_handler)

                # Run net with proper signal handling
                try:
                    net_task = asyncio.create_task(net_run())
                    stop_task = asyncio.create_task(stop_event.wait())

                    # Wait for either net to complete or stop signal
                    done, pending = await asyncio.wait(
                        [net_task, stop_task], return_when=asyncio.FIRST_COMPLETED
                    )

                    # Cancel pending tasks
                    for task in pending:
                        task.cancel()
                        try:
                            await task
                        except asyncio.CancelledError:
                            pass

                    # Check if we stopped due to signal
                    if stop_task in done:
                        console.print("[green]✓[/green] Node stopped gracefully")
                except asyncio.CancelledError:
                    console.print("[yellow]Cancelled[/yellow]")
            finally:
                sys.argv = original_argv
        else:
            # Run from source directory using the detected python_exe
            console.print(f"[green]✓[/green] Using hanzo/net from {net_path}")
            if python_exe != sys.executable:
                console.print(f"[green]✓[/green] Using hanzo/net venv")
            else:
                console.print("[yellow]⚠[/yellow] Using system Python")

            # Change to net directory and run
            original_cwd = os.getcwd()
            try:
                os.chdir(net_path)

                # Set up environment
                env = os.environ.copy()
                if models:
                    env["NET_MODELS"] = ",".join(models)
                if name:
                    env["NET_NODE_NAME"] = name
                env["PYTHONPATH"] = (
                    os.path.join(net_path, "src") + ":" + env.get("PYTHONPATH", "")
                )

                console.print(f"\n[green]✓[/green] Starting net node")
                console.print(f"  Port: {port}")
                console.print(
                    f"  Models: {', '.join(models) if models else 'auto-detect'}"
                )
                console.print("\n[bold green]Hanzo Net is running![/bold green]")
                console.print("WebUI: http://localhost:52415")
                console.print("API: http://localhost:52415/v1/chat/completions")
                console.print("\nPress Ctrl+C to stop\n")

                # Build command line args
                cmd_args = [python_exe, "-m", "net.main"]
                if port != 52415:
                    cmd_args.extend(["--chatgpt-api-port", str(port)])
                if name:
                    cmd_args.extend(["--node-id", name])
                if network != "local":
                    cmd_args.extend(["--discovery-module", network])
                if models:
                    cmd_args.extend(["--default-model", models[0]])

                # Run net command with detected python in a more signal-friendly way
                # Create new process group for better signal handling
                process = subprocess.Popen(
                    cmd_args,
                    env=env,
                    preexec_fn=os.setsid if hasattr(os, "setsid") else None,
                )

                # Set up signal handlers to forward to subprocess group
                def signal_handler(signum, frame):
                    if process.poll() is None:  # Process is still running
                        console.print("\n[yellow]Stopping hanzo net...[/yellow]")
                        try:
                            # Send signal to entire process group
                            if hasattr(os, "killpg"):
                                os.killpg(os.getpgid(process.pid), signal.SIGTERM)
                            else:
                                process.terminate()
                            process.wait(timeout=5)  # Wait up to 5 seconds
                        except subprocess.TimeoutExpired:
                            console.print("[yellow]Force stopping...[/yellow]")
                            if hasattr(os, "killpg"):
                                os.killpg(os.getpgid(process.pid), signal.SIGKILL)
                            else:
                                process.kill()
                            process.wait()
                        except ProcessLookupError:
                            pass  # Process already terminated
                    raise KeyboardInterrupt

                # Register signal handlers
                signal.signal(signal.SIGINT, signal_handler)
                signal.signal(signal.SIGTERM, signal_handler)

                # Wait for process to complete
                returncode = process.wait()

                if returncode != 0 and returncode != -2:  # -2 is Ctrl+C
                    console.print(f"[red]Net exited with code {returncode}[/red]")

            finally:
                os.chdir(original_cwd)

    except KeyboardInterrupt:
        console.print("\n[yellow]Shutting down node...[/yellow]")
        console.print("[green]✓[/green] Node stopped")
    except Exception as e:
        console.print(f"[red]Error starting compute node: {e}[/red]")


@cli.command()
@click.pass_context
def dashboard(ctx):
    """Open interactive dashboard."""
    from .interactive.dashboard import run_dashboard

    run_dashboard()


def main():
    """Main entry point."""
    try:
        cli(auto_envvar_prefix="HANZO")
    except KeyboardInterrupt:
        console.print("\n[yellow]Interrupted by user[/yellow]")
        sys.exit(0)
    except Exception as e:
        console.print(f"[red]Error: {e}[/red]")
        sys.exit(1)


if __name__ == "__main__":
    main()



"""Interactive REPL for Hanzo CLI."""

from typing import Optional
from pathlib import Path

from rich.console import Console
from rich.markdown import Markdown
from prompt_toolkit import PromptSession
from prompt_toolkit.history import FileHistory
from prompt_toolkit.completion import WordCompleter
from prompt_toolkit.auto_suggest import AutoSuggestFromHistory


class HanzoREPL:
    """Interactive REPL for Hanzo CLI."""

    def __init__(self, console: Optional[Console] = None):
        self.console = console or Console()
        self.session = PromptSession(
            history=FileHistory(".hanzo_repl_history"),
            auto_suggest=AutoSuggestFromHistory(),
        )
        self.commands = {
            "help": self.show_help,
            "exit": self.exit_repl,
            "quit": self.exit_repl,
            "clear": self.clear_screen,
            "status": self.show_status,
        }
        self.running = False

    async def run(self):
        """Run the REPL."""
        self.running = True
        # Don't print welcome message here since it's already printed in cli.py

        # Set up command completer
        completer = WordCompleter(
            list(self.commands.keys()) + ["chat", "agent", "cluster", "mcp", "network"],
            ignore_case=True,
        )

        while self.running:
            try:
                # Get input
                command = await self.session.prompt_async(
                    "hanzo> ", completer=completer
                )

                if not command.strip():
                    continue

                # Parse command
                parts = command.strip().split(maxsplit=1)
                cmd = parts[0].lower()
                args = parts[1] if len(parts) > 1 else ""

                # Execute command
                if cmd in self.commands:
                    await self.commands[cmd](args)
                else:
                    await self.execute_command(cmd, args)

            except KeyboardInterrupt:
                continue
            except EOFError:
                break
            except Exception as e:
                self.console.print(f"[red]Error: {e}[/red]")

    async def show_help(self, args: str = ""):
        """Show help message."""
        help_text = """
# Hanzo Interactive Mode

## Built-in Commands:
- `help` - Show this help message
- `exit/quit` - Exit interactive mode
- `clear` - Clear the screen
- `status` - Show system status

## CLI Commands:
All Hanzo CLI commands are available:
- `chat <message>` - Chat with AI
- `agent start` - Start an agent
- `cluster status` - Check cluster status
- `mcp tools` - List MCP tools
- `network agents` - List network agents

## Examples:
```
hanzo> chat How do I create a Python web server?
hanzo> agent list
hanzo> cluster start --models llama-3.2-3b
hanzo> mcp run read_file --arg path=README.md
```

## Tips:
- Use Tab for command completion
- Use ↑/↓ for command history
- Use Ctrl+R for reverse search
"""
        self.console.print(Markdown(help_text))

    def exit_repl(self, args: str = ""):
        """Exit the REPL."""
        self.running = False
        self.console.print("\n[yellow]Goodbye![/yellow]")

    def clear_screen(self, args: str = ""):
        """Clear the screen."""
        self.console.clear()

    async def show_status(self, args: str = ""):
        """Show system status."""
        status = {
            "cluster": await self.check_cluster_status(),
            "agents": await self.count_agents(),
            "auth": self.check_auth_status(),
        }

        self.console.print("[cyan]System Status:[/cyan]")
        self.console.print(f"  Cluster: {status['cluster']}")
        self.console.print(f"  Agents: {status['agents']}")
        self.console.print(f"  Auth: {status['auth']}")

    async def execute_command(self, cmd: str, args: str):
        """Execute a CLI command."""
        # Import here to avoid circular imports
        import sys

        import click

        from .. import cli

        # Build command line
        argv = [cmd]
        if args:
            import shlex

            argv.extend(shlex.split(args))

        # Create a new context
        try:
            # Save original argv
            orig_argv = sys.argv
            sys.argv = ["hanzo"] + argv

            # Execute command
            ctx = click.Context(cli.cli)
            cli.cli.invoke(ctx)

        except SystemExit:
            # Catch exit from commands
            pass
        except Exception as e:
            self.console.print(f"[red]Command error: {e}[/red]")
        finally:
            # Restore argv
            sys.argv = orig_argv

    async def check_cluster_status(self) -> str:
        """Check if cluster is running."""
        try:
            import httpx

            async with httpx.AsyncClient() as client:
                response = await client.get("http://localhost:8000/health", timeout=1.0)
                return "running" if response.status_code == 200 else "not responding"
        except Exception:
            return "not running"

    async def count_agents(self) -> int:
        """Count running agents."""
        # This would check actual agent status
        return 0

    def check_auth_status(self) -> str:
        """Check authentication status."""
        import os

        if os.environ.get("HANZO_API_KEY"):
            return "authenticated (API key)"
        elif (Path.home() / ".hanzo" / "auth.json").exists():
            return "authenticated (saved)"
        else:
            return "not authenticated"



"""Authentication commands."""

import os
from pathlib import Path

import click
from rich.prompt import Prompt

from ..utils.output import console


@click.group(name="auth")
def auth_group():
    """Manage Hanzo authentication."""
    pass


@auth_group.command()
@click.option("--email", "-e", help="Email address")
@click.option("--password", "-p", help="Password (not recommended, use prompt)")
@click.option("--api-key", "-k", help="API key for direct authentication")
@click.option("--sso", is_flag=True, help="Use SSO authentication")
@click.pass_context
async def login(ctx, email: str, password: str, api_key: str, sso: bool):
    """Login to Hanzo AI."""
    try:
        from hanzoai.auth import HanzoAuth
    except ImportError:
        console.print("[red]Error:[/red] hanzoai not installed")
        console.print("Install with: pip install hanzo")
        return

    auth = HanzoAuth()

    try:
        if api_key:
            # Direct API key authentication
            console.print("Authenticating with API key...")
            result = await auth.login_with_api_key(api_key)
        elif sso:
            # SSO authentication via browser
            console.print("Opening browser for SSO login...")
            console.print("If browser doesn't open, visit: https://iam.hanzo.ai/login")
            result = await auth.login_with_sso()
        else:
            # Email/password authentication
            if not email:
                email = Prompt.ask("Email")
            if not password:
                password = Prompt.ask("Password", password=True)

            console.print("Authenticating...")
            result = await auth.login(email, password)

        # Save credentials
        config_dir = Path.home() / ".hanzo"
        config_dir.mkdir(exist_ok=True)

        config_file = config_dir / "auth.json"
        await auth.save_credentials(config_file)

        # Also set environment variable if API key is available
        if api_key or result.get("api_key"):
            key = api_key or result.get("api_key")
            console.print(f"\n[dim]To use in environment:[/dim]")
            console.print(f"export HANZO_API_KEY={key}")

        console.print(f"[green]✓[/green] Logged in as {result.get('email', 'user')}")

        # Check organization
        if org := result.get("organization"):
            console.print(f"  Organization: {org}")

        # Check permissions
        if permissions := result.get("permissions"):
            console.print(f"  Permissions: {', '.join(permissions)}")

    except Exception as e:
        console.print(f"[red]Login failed: {e}[/red]")


@auth_group.command()
@click.pass_context
async def logout(ctx):
    """Logout from Hanzo AI."""
    try:
        from hanzoai.auth import HanzoAuth
    except ImportError:
        console.print("[red]Error:[/red] hanzoai not installed")
        return

    auth = HanzoAuth()

    try:
        await auth.logout()

        # Remove saved credentials
        config_file = Path.home() / ".hanzo" / "auth.json"
        if config_file.exists():
            config_file.unlink()

        console.print("[green]✓[/green] Logged out successfully")

    except Exception as e:
        console.print(f"[red]Logout failed: {e}[/red]")


@auth_group.command()
@click.pass_context
async def status(ctx):
    """Show authentication status."""
    # Check environment variable
    if api_key := os.environ.get("HANZO_API_KEY"):
        console.print("[green]✓[/green] HANZO_API_KEY environment variable set")
        console.print(f"  Key: {api_key[:8]}...{api_key[-4:]}")

    # Check saved credentials
    config_file = Path.home() / ".hanzo" / "auth.json"
    if config_file.exists():
        try:
            from hanzoai.auth import HanzoAuth

            auth = HanzoAuth()
            creds = await auth.load_credentials(config_file)

            console.print("[green]✓[/green] Saved credentials found")
            if email := creds.get("email"):
                console.print(f"  Email: {email}")
            if org := creds.get("organization"):
                console.print(f"  Organization: {org}")

            # Verify credentials are still valid
            with console.status("Verifying credentials..."):
                try:
                    user_info = await auth.get_user_info()
                    console.print("[green]✓[/green] Credentials are valid")

                    # Show usage stats if available
                    if usage := user_info.get("usage"):
                        console.print("\n[cyan]Usage:[/cyan]")
                        console.print(f"  API calls: {usage.get('api_calls', 0)}")
                        console.print(f"  Tokens: {usage.get('tokens', 0)}")
                        if quota := usage.get("quota"):
                            console.print(
                                f"  Quota: {usage.get('tokens', 0)} / {quota}"
                            )
                except Exception:
                    console.print("[yellow]![/yellow] Credentials may be expired")
                    console.print("Run 'hanzo auth login' to refresh")

        except Exception as e:
            console.print(f"[red]Error reading credentials: {e}[/red]")
    else:
        console.print("[yellow]![/yellow] Not logged in")
        console.print("Run 'hanzo auth login' to authenticate")


@auth_group.command()
@click.option("--name", "-n", required=True, help="API key name")
@click.option(
    "--permissions", "-p", multiple=True, help="Permissions (e.g., read, write, admin)"
)
@click.option("--expires", "-e", help="Expiration (e.g., 30d, 1y, never)")
@click.pass_context
async def create_key(ctx, name: str, permissions: tuple, expires: str):
    """Create a new API key."""
    try:
        from hanzoai.auth import HanzoAuth
    except ImportError:
        console.print("[red]Error:[/red] hanzoai not installed")
        return

    auth = HanzoAuth()

    # Ensure authenticated
    if not await auth.is_authenticated():
        console.print("[red]Error:[/red] Not authenticated")
        console.print("Run 'hanzo auth login' first")
        return

    with console.status(f"Creating API key '{name}'..."):
        try:
            result = await auth.create_api_key(
                name=name,
                permissions=list(permissions) if permissions else None,
                expires=expires,
            )

            key = result.get("key")
            console.print(f"[green]✓[/green] Created API key: {name}")
            console.print(
                f"\n[yellow]Save this key - it won't be shown again:[/yellow]"
            )
            console.print(f"{key}")
            console.print(f"\nTo use:")
            console.print(f"export HANZO_API_KEY={key}")

        except Exception as e:
            console.print(f"[red]Failed to create key: {e}[/red]")


@auth_group.command()
@click.pass_context
async def list_keys(ctx):
    """List your API keys."""
    try:
        from hanzoai.auth import HanzoAuth
    except ImportError:
        console.print("[red]Error:[/red] hanzoai not installed")
        return

    auth = HanzoAuth()

    # Ensure authenticated
    if not await auth.is_authenticated():
        console.print("[red]Error:[/red] Not authenticated")
        console.print("Run 'hanzo auth login' first")
        return

    with console.status("Loading API keys..."):
        try:
            keys = await auth.list_api_keys()

            if keys:
                from rich.table import Table

                table = Table(title="API Keys")
                table.add_column("Name", style="cyan")
                table.add_column("Created", style="green")
                table.add_column("Last Used", style="yellow")
                table.add_column("Permissions", style="blue")
                table.add_column("Status", style="magenta")

                for key in keys:
                    table.add_row(
                        key.get("name", "unknown"),
                        key.get("created_at", "unknown"),
                        key.get("last_used", "never"),
                        ", ".join(key.get("permissions", [])),
                        key.get("status", "active"),
                    )

                console.print(table)
            else:
                console.print("[yellow]No API keys found[/yellow]")
                console.print("Create one with: hanzo auth create-key")

        except Exception as e:
            console.print(f"[red]Failed to list keys: {e}[/red]")


@auth_group.command()
@click.argument("name")
@click.pass_context
async def revoke_key(ctx, name: str):
    """Revoke an API key."""
    try:
        from hanzoai.auth import HanzoAuth
    except ImportError:
        console.print("[red]Error:[/red] hanzoai not installed")
        return

    auth = HanzoAuth()

    # Ensure authenticated
    if not await auth.is_authenticated():
        console.print("[red]Error:[/red] Not authenticated")
        console.print("Run 'hanzo auth login' first")
        return

    if click.confirm(f"Revoke API key '{name}'?"):
        with console.status(f"Revoking key '{name}'..."):
            try:
                await auth.revoke_api_key(name)
                console.print(f"[green]✓[/green] Revoked API key: {name}")
            except Exception as e:
                console.print(f"[red]Failed to revoke key: {e}[/red]")


@auth_group.command()
@click.pass_context
async def whoami(ctx):
    """Show current user information."""
    try:
        from hanzoai.auth import HanzoAuth
    except ImportError:
        console.print("[red]Error:[/red] hanzoai not installed")
        return

    auth = HanzoAuth()

    # Check if authenticated
    if not await auth.is_authenticated():
        console.print("[yellow]Not authenticated[/yellow]")

        # Check if API key is set
        if os.environ.get("HANZO_API_KEY"):
            console.print("HANZO_API_KEY is set but may be invalid")
        else:
            console.print("Run 'hanzo auth login' to authenticate")
        return

    with console.status("Loading user information..."):
        try:
            user = await auth.get_user_info()

            console.print(f"[cyan]User Information:[/cyan]")
            console.print(f"  ID: {user.get('id', 'unknown')}")
            console.print(f"  Email: {user.get('email', 'unknown')}")
            console.print(f"  Name: {user.get('name', 'unknown')}")

            if org := user.get("organization"):
                console.print(f"\n[cyan]Organization:[/cyan]")
                console.print(f"  Name: {org.get('name', 'unknown')}")
                console.print(f"  Role: {org.get('role', 'member')}")

            if teams := user.get("teams"):
                console.print(f"\n[cyan]Teams:[/cyan]")
                for team in teams:
                    console.print(f"  • {team}")

            if perms := user.get("permissions"):
                console.print(f"\n[cyan]Permissions:[/cyan]")
                for perm in sorted(perms):
                    console.print(f"  • {perm}")

        except Exception as e:
            console.print(f"[red]Error: {e}[/red]")
